"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkerManager = void 0;
const bullmq_1 = require("bullmq");
const RedisClient_1 = require("../utils/RedisClient");
const NotifierRegistry_1 = require("./NotifierRegistry");
const ResponseTrackers_1 = require("../utils/ResponseTrackers");
const Logger_1 = __importDefault(require("../utils/Logger"));
const SimulationManager_1 = require("./SimulationManager");
class WorkerManager {
    config;
    worker;
    queue;
    redisInstance;
    constructor(config) {
        this.config = config;
        this.redisInstance = RedisClient_1.RedisClient.getInstance();
        this.queue = new bullmq_1.Queue(this.config.queueName, {
            connection: this.redisInstance,
        });
        this.worker = new bullmq_1.Worker(this.config.queueName, this.jobProcessor.bind(this), {
            connection: {
                ...(this.redisInstance.options || {}),
                maxRetriesPerRequest: null,
                enableReadyCheck: false,
            },
            concurrency: this.config.concurrency || 10,
        });
        this.worker.on("active", async (job) => {
            Logger_1.default.log(`🚀 Job ${job.id} started processing.`);
            if (this.config.onStart) {
                try {
                    await this.config.onStart(job);
                }
                catch (error) {
                    Logger_1.default.error(`❌ Error in user-provided onStart callback for job ${job.id}:`, error);
                }
            }
        });
        this.worker.on("completed", async (job) => {
            if (!job) {
                Logger_1.default.warn("Received 'completed' event with undefined job.");
                return;
            }
            Logger_1.default.log(`✅ Job ${job.id} completed.`);
            if (this.config.onComplete) {
                try {
                    const stats = await (0, ResponseTrackers_1.getNotificationStats)(this.config.trackingKey || "notifications:stats");
                    await this.config.onComplete(job, stats);
                }
                catch (error) {
                    Logger_1.default.error(`❌ Error in user-provided onComplete callback for job ${job.id}:`, error);
                }
            }
        });
        this.worker.on("failed", (job, error) => {
            Logger_1.default.error(`❌ Job failed (${job?.id}):`, error);
        });
        this.worker.on("drained", async () => {
            let { active, waiting } = await this.queue.getJobCounts();
            while (active > 0 || waiting > 0) {
                Logger_1.default.log(`⏳ Still ${active} active jobs & ${waiting} waiting jobs...`);
                await new Promise((res) => setTimeout(res, 1000));
                ({ active, waiting } = await this.queue.getJobCounts());
            }
            Logger_1.default.log("✅ No active/waiting jobs remain. Queue is fully processed.");
            if (this.config.onDrained) {
                await this.config.onDrained();
            }
        });
        Logger_1.default.log(`🚀 Worker started listening on queue "${this.config.queueName}"`);
    }
    async jobProcessor(job) {
        const { userIds, channel, meta, trackResponses, trackingKey } = job.data;
        if (await new SimulationManager_1.SimulationManager(this.redisInstance).isSimulationEnabled()) {
            Logger_1.default.log(`🔄 [Simulated] Processing job ${job.id} for channel ${channel}.`);
            const simulatedResponse = userIds.map((id) => ({
                status: "success",
                recipient: id,
                response: "Simulated delivery",
            }));
            if (trackResponses && simulatedResponse) {
                const resolvedTrackingKey = trackingKey || this.config.trackingKey || "notifications:stats";
                await (0, ResponseTrackers_1.trackNotificationResponse)(resolvedTrackingKey, simulatedResponse);
            }
            return;
        }
        const notifier = NotifierRegistry_1.NotifierRegistry.get(channel);
        try {
            const response = await notifier.send(userIds, meta);
            if (trackResponses && response) {
                await (0, ResponseTrackers_1.trackNotificationResponse)(trackingKey, response);
            }
            Logger_1.default.log(`Notifier sent successfully for job ${job.id}, channel ${channel}.`);
        }
        catch (error) {
            Logger_1.default.error(`❌ Notification failed: ${error.message}`);
            if (trackResponses) {
                await (0, ResponseTrackers_1.trackNotificationResponse)(trackingKey, {
                    success: false,
                    error: error.message,
                });
            }
            throw error;
        }
    }
    async close() {
        Logger_1.default.log(`Closing worker and queue for "${this.config.queueName}"...`);
        await this.worker.close();
        await this.queue.close();
        Logger_1.default.log(`Worker and queue for "${this.config.queueName}" closed.`);
    }
}
exports.WorkerManager = WorkerManager;
//# sourceMappingURL=WorkerManager.js.map